@@ -1,19 +1,11 @@
-// Copyright (C) 2013-2015  Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2013-2017 Internet Systems Consortium, Inc. ("ISC")
 //
-// Permission to use, copy, modify, and/or distribute this software for any
-// purpose with or without fee is hereby granted, provided that the above
-// copyright notice and this permission notice appear in all copies.
-//
-// THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES WITH
-// REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
-// AND FITNESS.  IN NO EVENT SHALL ISC BE LIABLE FOR ANY SPECIAL, DIRECT,
-// INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
-// LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
-// OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
-// PERFORMANCE OF THIS SOFTWARE.
+// This Source Code Form is subject to the terms of the Mozilla Public
+// License, v. 2.0. If a copy of the MPL was not distributed with this
+// file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 #include <config.h>
-
+#include <asiolink/asio_wrapper.h>
 #include <asiolink/io_service.h>
 #include <asiolink/interval_timer.h>
 #include <d2/nc_trans.h>
@@ -24,10 +16,6 @@
 #include <util/buffer.h>
 #include <nc_test_utils.h>
 
-#include <boost/asio/ip/udp.hpp>
-#include <boost/asio/socket_base.hpp>
-#include <boost/asio.hpp>
-
 #include <boost/function.hpp>
 #include <boost/bind.hpp>
 #include <boost/date_time/posix_time/posix_time.hpp>
@@ -36,6 +24,7 @@
 using namespace std;
 using namespace isc;
 using namespace isc::d2;
+using namespace isc::util;
 using namespace boost::posix_time;
 
 namespace {
@@ -77,7 +66,7 @@ public:
 
     /// @brief DNSClient callback
     /// Overrides the callback in NameChangeTranscation to allow testing
-    /// sendUpdate without incorporating exectution of the state model
+    /// sendUpdate without incorporating execution of the state model
     /// into the test.
     /// It sets the DNS status update and posts IO_COMPLETED_EVT as does
     /// the normal callback.
@@ -324,7 +313,7 @@ public:
     /// @brief Builds and then sends an update request
     ///
     /// This method is used to build and send and update request. It is used
-    /// in conjuction with FauxServer to test various message response
+    /// in conjunction with FauxServer to test various message response
     /// scenarios.
     /// @param name_change Transaction under test
     /// @param run_time Maximum time to permit IO processing to run before
@@ -835,7 +824,7 @@ TEST_F(NameChangeTransactionTest, successfulUpdateTest) {
     EXPECT_TRUE(name_change->getForwardChangeCompleted());
 }
 
-/// @brief Tests the ability to use startTransaction to initate the state
+/// @brief Tests the ability to use startTransaction to initiate the state
 /// model execution, and DNSClient callback, operator(), to resume the
 /// model with a update failure outcome.
 TEST_F(NameChangeTransactionTest, failedUpdateTest) {
@@ -1054,7 +1043,7 @@ TEST_F(NameChangeTransactionTest, tsigUnsignedResponse) {
     FauxServer server(*io_service_, *(name_change->getCurrentServer()));
     server.receive (FauxServer::USE_RCODE, dns::Rcode::NOERROR());
 
-    // Do the udpate.
+    // Do the update.
     ASSERT_NO_FATAL_FAILURE(doOneExchange(name_change));
 
     // Verify that next event is IO_COMPLETED_EVT and DNS status is
@@ -1085,7 +1074,7 @@ TEST_F(NameChangeTransactionTest, tsigInvalidResponse) {
     FauxServer server(*io_service_, *(name_change->getCurrentServer()));
     server.receive (FauxServer::INVALID_TSIG, dns::Rcode::NOERROR());
 
-    // Do the udpate.
+    // Do the update.
     ASSERT_NO_FATAL_FAILURE(doOneExchange(name_change));
 
     // Verify that next event is IO_COMPLETED_EVT and DNS status is
@@ -1135,7 +1124,7 @@ TEST_F(NameChangeTransactionTest, tsigUnexpectedSignedResponse) {
     EXPECT_EQ("response.example.com.", zone->getName().toText());
 }
 
-/// @brief Tests that a TSIG udpate succeeds when client and server both use
+/// @brief Tests that a TSIG update succeeds when client and server both use
 /// the right key.  Runs the test for all supported algorithms.
 TEST_F(NameChangeTransactionTest, tsigAllValid) {
     std::vector<std::string>algorithms;
@@ -1206,6 +1195,15 @@ TEST_F(NameChangeTransactionTest, prepNewRequest) {
     // valid input.
     ASSERT_NO_THROW(request = name_change->prepNewRequest(forward_domain_));
     checkZone(request, forward_domain_->getName());
+
+    // The query id is random so 0 is not impossible
+    for (unsigned i = 0; i < 10; ++i) {
+        if (request->getId() == 0) {
+            request = name_change->prepNewRequest(forward_domain_);
+        }
+    }
+
+    EXPECT_NE(0, request->getId());
 }
 
 /// @brief Tests the addLeaseAddressRData method
