@@ -1,16 +1,8 @@
-// Copyright (C) 2015 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2015-2017 Internet Systems Consortium, Inc. ("ISC")
 //
-// Permission to use, copy, modify, and/or distribute this software for any
-// purpose with or without fee is hereby granted, provided that the above
-// copyright notice and this permission notice appear in all copies.
-//
-// THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES WITH
-// REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
-// AND FITNESS.  IN NO EVENT SHALL ISC BE LIABLE FOR ANY SPECIAL, DIRECT,
-// INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
-// LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
-// OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
-// PERFORMANCE OF THIS SOFTWARE.
+// This Source Code Form is subject to the terms of the Mozilla Public
+// License, v. 2.0. If a copy of the MPL was not distributed with this
+// file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 #include <config.h>
 #include <dhcp/duid.h>
@@ -56,7 +48,7 @@ const unsigned int TEST_LEASES_NUM = 100;
 struct LowerBound {
     /// @brief Constructor.
     ///
-    /// @param lower_bound Interger value wrapped by the structure.
+    /// @param lower_bound Integer value wrapped by the structure.
     explicit LowerBound(const size_t lower_bound)
         : lower_bound_(lower_bound) { };
 
@@ -78,7 +70,7 @@ struct LowerBound {
 struct UpperBound {
     /// @brief Constructor.
     ///
-    /// @param lower_bound Interger value wrapped by the structure.
+    /// @param lower_bound Integer value wrapped by the structure.
     explicit UpperBound(const size_t upper_bound)
         : upper_bound_(upper_bound) { };
 
@@ -106,7 +98,7 @@ std::string callout_argument_name("lease4");
 /// - it processes multiple leases,
 /// - leases are processed in certain order,
 /// - number of processed leases may be limited by the parameters,
-/// - maxium duration of the lease reclamation routine may be limited,
+/// - maximum duration of the lease reclamation routine may be limited,
 /// - reclaimed leases may be marked as reclaimed or deleted,
 /// - DNS records for some of the leases must be removed when the lease
 ///   is reclaimed and DNS updates are enabled,
@@ -312,7 +304,7 @@ public:
     /// @brief Wrapper method running lease reclamation routine.
     ///
     /// @param max_leases Maximum number of leases to be reclaimed.
-    /// @param timeout Maximum amount of time that the reclaimation routine
+    /// @param timeout Maximum amount of time that the reclamation routine
     /// may be processing expired leases, expressed in seconds.
     /// @param remove_lease A boolean value indicating if the lease should
     /// be removed when it is reclaimed (if true) or it should be left in the
@@ -791,7 +783,7 @@ public:
                 << "check failed for i = " << i;
 
 
-            // At early stages of iterations, there should be conitnuous
+            // At early stages of iterations, there should be continuous
             // group of leases (expired and not expired) which haven't been
             // reclaimed.
             if (reclaimed_lower_bound > 0) {
@@ -846,7 +838,7 @@ public:
         EXPECT_TRUE(testLeases(&dnsUpdateGeneratedForLease, &oddLeaseIndex));
     }
 
-    /// @brief This test verfies that callouts are executed for each expired
+    /// @brief This test verifies that callouts are executed for each expired
     /// lease when installed.
     void testReclaimExpiredLeasesHooks() {
         for (unsigned int i = 0; i < TEST_LEASES_NUM; ++i) {
@@ -855,7 +847,7 @@ public:
             }
         }
 
-        vector<string> libraries; // no libraries at this time
+        HookLibsCollection libraries; // no libraries at this time
         HooksManager::loadLibraries(libraries);
 
         // Install a callout: lease4_expire or lease6_expire.
@@ -876,7 +868,7 @@ public:
         EXPECT_TRUE(testLeases(&leaseNotReclaimed, &oddLeaseIndex));
     }
 
-    /// @brief This test verfies that callouts are executed for each expired
+    /// @brief This test verifies that callouts are executed for each expired
     /// lease and that the lease is not reclaimed when skip flag is set.
     void testReclaimExpiredLeasesHooksWithSkip() {
         for (unsigned int i = 0; i < TEST_LEASES_NUM; ++i) {
@@ -885,7 +877,7 @@ public:
             }
         }
 
-        vector<string> libraries; // no libraries at this time
+        HookLibsCollection libraries; // no libraries at this time
         HooksManager::loadLibraries(libraries);
 
         // Install a callout: lease4_expire or lease6_expire.
@@ -913,7 +905,7 @@ public:
             expire(i, 2000 - i);
         }
 
-        vector<string> libraries;
+        HookLibsCollection libraries;
         HooksManager::loadLibraries(libraries);
 
         // Install a callout: lease4_expire or lease6_expire. Each callout
@@ -1008,11 +1000,11 @@ public:
     ///
     /// This method works for both v4 and v6. Just make sure the correct
     /// statistic name is passed. This is the name of the assigned addresses,
-    /// that is expected to be decreased once the reclaimation procedure
+    /// that is expected to be decreased once the reclamation procedure
     /// is complete.
     ///
     /// @param stat_name name of the statistic for assigned addresses statistic
-    ///        ("assgined-addresses" for both v4 and "assigned-nas" for v6)
+    ///        ("assigned-addresses" for both v4 and "assigned-nas" for v6)
     void testReclaimDeclinedStats(const std::string& stat_name) {
 
         // Leases by default all belong to subnet_id_ = 1. Let's count the
@@ -1080,8 +1072,8 @@ public:
 
         // subnet[X].assigned-addresses should go down. Between the time
         // of DHCPDECLINE(v4)/DECLINE(v6) reception and declined expired lease
-        // reclaimation, we count this address as assigned-addresses. We decrease
-        // assigned-addresses(v4)/assgined-nas(v6) when we reclaim the lease,
+        // reclamation, we count this address as assigned-addresses. We decrease
+        // assigned-addresses(v4)/assigned-nas(v6) when we reclaim the lease,
         // not when the packet is received. For explanation, see Duplicate
         // Addresses (DHCPDECLINE support) (v4) or Duplicate Addresses (DECLINE
         // support) sections in the User's Guide or a comment in
@@ -1172,7 +1164,7 @@ public:
     /// @brief Wrapper method running lease reclamation routine.
     ///
     /// @param max_leases Maximum number of leases to be reclaimed.
-    /// @param timeout Maximum amount of time that the reclaimation routine
+    /// @param timeout Maximum amount of time that the reclamation routine
     /// may be processing expired leases, expressed in seconds.
     /// @param remove_lease A boolean value indicating if the lease should
     /// be removed when it is reclaimed (if true) or it should be left in the
@@ -1423,14 +1415,13 @@ ExpirationAllocEngine6Test::testReclaimReusedLeases(const uint16_t msg_type,
 
     for (unsigned int i = 0; i < TEST_LEASES_NUM; ++i) {
         // Build the context.
-        AllocEngine::ClientContext6 ctx(subnet, leases_[i]->duid_, 1,
-                                        leases_[i]->addr_,
-                                        Lease::TYPE_NA,
+        AllocEngine::ClientContext6 ctx(subnet, leases_[i]->duid_,
                                         false, false,
                                         leases_[i]->hostname_,
-                                        msg_type == DHCPV6_SOLICIT);
-        // Query is needed for logging purposes.
-        ctx.query_.reset(new Pkt6(msg_type, 0x1234));
+                                        msg_type == DHCPV6_SOLICIT,
+                                        Pkt6Ptr(new Pkt6(msg_type, 0x1234)));
+        ctx.currentIA().iaid_ = 1;
+        ctx.currentIA().hints_.push_back(std::make_pair(leases_[i]->addr_, 128));
 
         // Depending on the message type, we will call a different function.
         if (msg_type == DHCPV6_RENEW) {
@@ -1716,7 +1707,7 @@ public:
     /// @brief Wrapper method running lease reclamation routine.
     ///
     /// @param max_leases Maximum number of leases to be reclaimed.
-    /// @param timeout Maximum amount of time that the reclaimation routine
+    /// @param timeout Maximum amount of time that the reclamation routine
     /// may be processing expired leases, expressed in seconds.
     /// @param remove_lease A boolean value indicating if the lease should
     /// be removed when it is reclaimed (if true) or it should be left in the
@@ -1862,7 +1853,7 @@ void
 ExpirationAllocEngine4Test::setUniqueClientId(const uint16_t index) {
     std::ostringstream clientid_s;
     clientid_s << "AA:BB:" << std::setw(2) << std::setfill('0')
-        << (index >> 16) << ":" << std::setw(2) << std::setfill('0')
+        << (index >> 8) << ":" << std::setw(2) << std::setfill('0')
         << (index & 0x00FF);
     ClientIdPtr client_id(ClientId::fromText(clientid_s.str()));
     leases_[index]->client_id_ = client_id;
