@@ -1,16 +1,8 @@
-// Copyright (C) 2015 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2015-2017 Internet Systems Consortium, Inc. ("ISC")
 //
-// Permission to use, copy, modify, and/or distribute this software for any
-// purpose with or without fee is hereby granted, provided that the above
-// copyright notice and this permission notice appear in all copies.
-//
-// THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES WITH
-// REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
-// AND FITNESS.  IN NO EVENT SHALL ISC BE LIABLE FOR ANY SPECIAL, DIRECT,
-// INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
-// LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
-// OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
-// PERFORMANCE OF THIS SOFTWARE.
+// This Source Code Form is subject to the terms of the Mozilla Public
+// License, v. 2.0. If a copy of the MPL was not distributed with this
+// file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 #include <config.h>
 #include <dhcpsrv/tests/alloc_engine_utils.h>
@@ -52,6 +44,9 @@ public:
         callback_argument_names_.clear();
         callback_addr_original_ = IOAddress("::");
         callback_addr_updated_ = IOAddress("::");
+        callback_qry_pkt6_.reset();
+        callback_qry_options_copy_ = false;
+        callback_skip_ = 0;
     }
 
     /// callback that stores received callout name and received values
@@ -60,6 +55,7 @@ public:
 
         callback_name_ = string("lease6_select");
 
+        callout_handle.getArgument("query6", callback_qry_pkt6_);
         callout_handle.getArgument("subnet6", callback_subnet6_);
         callout_handle.getArgument("fake_allocation", callback_fake_allocation_);
         callout_handle.getArgument("lease6", callback_lease6_);
@@ -67,6 +63,12 @@ public:
         callback_addr_original_ = callback_lease6_->addr_;
 
         callback_argument_names_ = callout_handle.getArgumentNames();
+
+        if (callback_qry_pkt6_) {
+            callback_qry_options_copy_ =
+                callback_qry_pkt6_->isCopyRetrievedOptions();
+        }
+
         return (0);
     }
 
@@ -90,6 +92,21 @@ public:
         return (0);
     }
 
+    /// callback that return next step skip status
+    static int
+    lease6_select_skip_callout(CalloutHandle& callout_handle) {
+
+        // Let's call the basic callout, so it can record all parameters
+        lease6_select_callout(callout_handle);
+
+        // Count the call
+        callback_skip_++;
+
+        callout_handle.setStatus(CalloutHandle::NEXT_STEP_SKIP);
+
+        return (0);
+    }
+
     // Values to be used in callout to override lease6 content
     static const IOAddress addr_override_;
     static const uint32_t t1_override_;
@@ -107,9 +124,14 @@ public:
     static Lease6Ptr callback_lease6_;
     static bool callback_fake_allocation_;
     static vector<string> callback_argument_names_;
+    static Pkt6Ptr callback_qry_pkt6_;
+    static bool callback_qry_options_copy_;
+
+    // Counter for next step skip (should be not retried)
+    static unsigned callback_skip_;
 };
 
-// For some reason intialization within a class makes the linker confused.
+// For some reason initialization within a class makes the linker confused.
 // linker complains about undefined references if they are defined within
 // the class declaration.
 const IOAddress HookAllocEngine6Test::addr_override_("2001:db8::abcd");
@@ -126,6 +148,10 @@ Subnet6Ptr HookAllocEngine6Test::callback_subnet6_;
 Lease6Ptr HookAllocEngine6Test::callback_lease6_;
 bool HookAllocEngine6Test::callback_fake_allocation_;
 vector<string> HookAllocEngine6Test::callback_argument_names_;
+Pkt6Ptr HookAllocEngine6Test::callback_qry_pkt6_;
+bool HookAllocEngine6Test::callback_qry_options_copy_;
+
+unsigned HookAllocEngine6Test::callback_skip_;
 
 // This test checks if the lease6_select callout is executed and expected
 // parameters as passed.
@@ -145,19 +171,19 @@ TEST_F(HookAllocEngine6Test, lease6_select) {
     ASSERT_TRUE(engine);
 
     // Initialize Hooks Manager
-    vector<string> libraries; // no libraries at this time
+    HookLibsCollection libraries; // no libraries at this time
     HooksManager::loadLibraries(libraries);
 
-    // Install pkt6_receive_callout
+    // Install lease6_select
     EXPECT_NO_THROW(HooksManager::preCalloutsLibraryHandle().registerCallout(
                         "lease6_select", lease6_select_callout));
 
-    CalloutHandlePtr callout_handle = HooksManager::createCalloutHandle();
-
     Lease6Ptr lease;
-    AllocEngine::ClientContext6 ctx(subnet_, duid_, iaid_, IOAddress("::"),
-                                    Lease::TYPE_NA, false, false, "", false);
-    ctx.callout_handle_ = callout_handle;
+    AllocEngine::ClientContext6 ctx(subnet_, duid_, false, false, "", false,
+                                    Pkt6Ptr(new Pkt6(DHCPV6_REQUEST, 1234)),
+                                    HooksManager::createCalloutHandle());
+    ctx.currentIA().iaid_ = iaid_;
+
     EXPECT_NO_THROW(lease = expectOneLease(engine->allocateLeases6(ctx)));
     // Check that we got a lease
     ASSERT_TRUE(lease);
@@ -173,6 +199,10 @@ TEST_F(HookAllocEngine6Test, lease6_select) {
     // Check that callouts were indeed called
     EXPECT_EQ("lease6_select", callback_name_);
 
+    // Check that query6 argument was set correctly
+    ASSERT_TRUE(callback_qry_pkt6_);
+    EXPECT_EQ(callback_qry_pkt6_.get(), ctx.query_.get());
+
     // Now check that the lease in LeaseMgr has the same parameters
     ASSERT_TRUE(callback_lease6_);
     detailCompareLease(callback_lease6_, from_mgr);
@@ -182,18 +212,21 @@ TEST_F(HookAllocEngine6Test, lease6_select) {
 
     EXPECT_FALSE(callback_fake_allocation_);
 
-    // Check if all expected parameters are reported. It's a bit tricky, because
-    // order may be different. If the test starts failing, because someone tweaked
-    // hooks engine, we'll have to implement proper vector matching (ignoring order)
+    // Check if all expected parameters are reported.  The order needs to be
+    // alphabetical to match the order returned by
+    // CallbackHandle::getAgrumentNames()
     vector<string> expected_argument_names;
     expected_argument_names.push_back("fake_allocation");
     expected_argument_names.push_back("lease6");
+    expected_argument_names.push_back("query6");
     expected_argument_names.push_back("subnet6");
 
     sort(callback_argument_names_.begin(), callback_argument_names_.end());
     sort(expected_argument_names.begin(), expected_argument_names.end());
 
     EXPECT_TRUE(callback_argument_names_ == expected_argument_names);
+
+    EXPECT_TRUE(callback_qry_options_copy_);
 }
 
 // This test checks if lease6_select callout is able to override the values
@@ -214,22 +247,19 @@ TEST_F(HookAllocEngine6Test, change_lease6_select) {
     ASSERT_TRUE(engine);
 
     // Initialize Hooks Manager
-    vector<string> libraries; // no libraries at this time
+    HookLibsCollection libraries; // no libraries at this time
     HooksManager::loadLibraries(libraries);
 
     // Install a callout
     EXPECT_NO_THROW(HooksManager::preCalloutsLibraryHandle().registerCallout(
                         "lease6_select", lease6_select_different_callout));
 
-    // Normally, dhcpv6_srv would passed the handle when calling allocateLeases6,
-    // but in tests we need to create it on our own.
-    CalloutHandlePtr callout_handle = HooksManager::createCalloutHandle();
-
     // Call allocateLeases6. Callouts should be triggered here.
     Lease6Ptr lease;
-    AllocEngine::ClientContext6 ctx(subnet_, duid_, iaid_, IOAddress("::"),
-                                    Lease::TYPE_NA, false, false, "", false);
-    ctx.callout_handle_ = callout_handle;
+    AllocEngine::ClientContext6 ctx(subnet_, duid_, false, false, "", false,
+                                    Pkt6Ptr(new Pkt6(DHCPV6_REQUEST, 1234)),
+                                    HooksManager::createCalloutHandle());
+    ctx.currentIA().iaid_ = iaid_;
     EXPECT_NO_THROW(lease = expectOneLease(engine->allocateLeases6(ctx)));
     // Check that we got a lease
     ASSERT_TRUE(lease);
@@ -254,6 +284,36 @@ TEST_F(HookAllocEngine6Test, change_lease6_select) {
     EXPECT_EQ(valid_override_, from_mgr->valid_lft_);
 }
 
+// This test checks if lease6_select callout can set the status to next
+// step skip without the engine to retry.
+TEST_F(HookAllocEngine6Test, skip_lease6_select) {
+
+    // Create allocation engine (hook names are registered in its ctor)
+    boost::scoped_ptr<AllocEngine> engine;
+    ASSERT_NO_THROW(engine.reset(new AllocEngine(AllocEngine::ALLOC_ITERATIVE, 100)));
+    ASSERT_TRUE(engine);
+
+    // Initialize Hooks Manager
+    HookLibsCollection libraries; // no libraries at this time
+    HooksManager::loadLibraries(libraries);
+
+    // Install a callout
+    EXPECT_NO_THROW(HooksManager::preCalloutsLibraryHandle().registerCallout(
+                        "lease6_select", lease6_select_skip_callout));
+
+    // Call allocateLeases6. Callouts should be triggered here.
+    Lease6Ptr lease;
+    AllocEngine::ClientContext6 ctx(subnet_, duid_, false, false, "", false,
+                                    Pkt6Ptr(new Pkt6(DHCPV6_REQUEST, 1234)),
+                                    HooksManager::createCalloutHandle());
+    ctx.currentIA().iaid_ = iaid_;
+    EXPECT_NO_THROW(lease = expectOneLease(engine->allocateLeases6(ctx)));
+    // Check that we got no lease
+    EXPECT_FALSE(lease);
+
+    // Check no retry was attempted
+    EXPECT_EQ(1, callback_skip_);
+}
 
 /// @brief helper class used in Hooks testing in AllocEngine4
 ///
@@ -283,6 +343,9 @@ public:
         callback_argument_names_.clear();
         callback_addr_original_ = IOAddress("::");
         callback_addr_updated_ = IOAddress("::");
+        callback_qry_pkt4_.reset();
+        callback_qry_options_copy_ = false;
+        callback_skip_ = 0;
     }
 
     /// callback that stores received callout name and received values
@@ -291,6 +354,7 @@ public:
 
         callback_name_ = string("lease4_select");
 
+        callout_handle.getArgument("query4", callback_qry_pkt4_);
         callout_handle.getArgument("subnet4", callback_subnet4_);
         callout_handle.getArgument("fake_allocation", callback_fake_allocation_);
         callout_handle.getArgument("lease4", callback_lease4_);
@@ -298,6 +362,12 @@ public:
         callback_addr_original_ = callback_lease4_->addr_;
 
         callback_argument_names_ = callout_handle.getArgumentNames();
+
+        if (callback_qry_pkt4_) {
+            callback_qry_options_copy_ =
+                callback_qry_pkt4_->isCopyRetrievedOptions();
+        }
+
         return (0);
     }
 
@@ -320,6 +390,21 @@ public:
         return (0);
     }
 
+    /// callback that return next step skip status
+    static int
+    lease4_select_skip_callout(CalloutHandle& callout_handle) {
+
+        // Let's call the basic callout, so it can record all parameters
+        lease4_select_callout(callout_handle);
+
+        // Count the call
+        callback_skip_++;
+
+        callout_handle.setStatus(CalloutHandle::NEXT_STEP_SKIP);
+
+        return (0);
+    }
+
     // Values to be used in callout to override lease4 content
     static const IOAddress addr_override_;
     static const uint32_t t1_override_;
@@ -336,9 +421,14 @@ public:
     static Lease4Ptr callback_lease4_;
     static bool callback_fake_allocation_;
     static vector<string> callback_argument_names_;
+    static Pkt4Ptr callback_qry_pkt4_;
+    static bool callback_qry_options_copy_;
+
+    // Counter for next step skip (should be not retried)
+    static unsigned callback_skip_;
 };
 
-// For some reason intialization within a class makes the linker confused.
+// For some reason initialization within a class makes the linker confused.
 // linker complains about undefined references if they are defined within
 // the class declaration.
 const IOAddress HookAllocEngine4Test::addr_override_("192.0.3.1");
@@ -354,6 +444,10 @@ Subnet4Ptr HookAllocEngine4Test::callback_subnet4_;
 Lease4Ptr HookAllocEngine4Test::callback_lease4_;
 bool HookAllocEngine4Test::callback_fake_allocation_;
 vector<string> HookAllocEngine4Test::callback_argument_names_;
+Pkt4Ptr HookAllocEngine4Test::callback_qry_pkt4_;
+bool HookAllocEngine4Test::callback_qry_options_copy_;
+
+unsigned HookAllocEngine4Test::callback_skip_;
 
 // This test checks if the lease4_select callout is executed and expected
 // parameters as passed.
@@ -374,20 +468,23 @@ TEST_F(HookAllocEngine4Test, lease4_select) {
     ASSERT_TRUE(engine);
 
     // Initialize Hooks Manager
-    vector<string> libraries; // no libraries at this time
+    HookLibsCollection libraries; // no libraries at this time
     HooksManager::loadLibraries(libraries);
 
-    // Install pkt4_receive_callout
+    // Install lease4_select
     EXPECT_NO_THROW(HooksManager::preCalloutsLibraryHandle().registerCallout(
                         "lease4_select", lease4_select_callout));
 
     CalloutHandlePtr callout_handle = HooksManager::createCalloutHandle();
 
-    Lease4Ptr lease = engine->allocateLease4(subnet_, clientid_, hwaddr_,
-                                               IOAddress("0.0.0.0"),
-                                               false, false, "",
-                                               false, callout_handle,
-                                               old_lease_);
+    AllocEngine::ClientContext4 ctx(subnet_, clientid_, hwaddr_,
+                                    IOAddress("0.0.0.0"),
+                                    false, false, "", false);
+    ctx.query_.reset(new Pkt4(DHCPREQUEST, 1234));
+    ctx.callout_handle_ = callout_handle;
+
+    Lease4Ptr lease = engine->allocateLease4(ctx);
+
     // Check that we got a lease
     ASSERT_TRUE(lease);
 
@@ -401,6 +498,10 @@ TEST_F(HookAllocEngine4Test, lease4_select) {
     // Check that callouts were indeed called
     EXPECT_EQ("lease4_select", callback_name_);
 
+    // Check that query4 argument was set correctly
+    ASSERT_TRUE(callback_qry_pkt4_);
+    EXPECT_EQ(callback_qry_pkt4_.get(), ctx.query_.get());
+
     // Now check that the lease in LeaseMgr has the same parameters
     ASSERT_TRUE(callback_lease4_);
     detailCompareLease(callback_lease4_, from_mgr);
@@ -410,14 +511,17 @@ TEST_F(HookAllocEngine4Test, lease4_select) {
 
     EXPECT_EQ(callback_fake_allocation_, false);
 
-    // Check if all expected parameters are reported. It's a bit tricky, because
-    // order may be different. If the test starts failing, because someone tweaked
-    // hooks engine, we'll have to implement proper vector matching (ignoring order)
+    // Check if all expected parameters are reported.  The order needs to be
+    // alphabetical to match the order returned by
+    // CallbackHandle::getAgrumentNames()
     vector<string> expected_argument_names;
     expected_argument_names.push_back("fake_allocation");
     expected_argument_names.push_back("lease4");
+    expected_argument_names.push_back("query4");
     expected_argument_names.push_back("subnet4");
     EXPECT_TRUE(callback_argument_names_ == expected_argument_names);
+
+    EXPECT_TRUE(callback_qry_options_copy_);
 }
 
 // This test checks if lease4_select callout is able to override the values
@@ -438,7 +542,7 @@ TEST_F(HookAllocEngine4Test, change_lease4_select) {
     ASSERT_TRUE(engine);
 
     // Initialize Hooks Manager
-    vector<string> libraries; // no libraries at this time
+    HookLibsCollection libraries; // no libraries at this time
     HooksManager::loadLibraries(libraries);
 
     // Install a callout
@@ -449,12 +553,15 @@ TEST_F(HookAllocEngine4Test, change_lease4_select) {
     // but in tests we need to create it on our own.
     CalloutHandlePtr callout_handle = HooksManager::createCalloutHandle();
 
+
+    AllocEngine::ClientContext4 ctx(subnet_, clientid_, hwaddr_, IOAddress("0.0.0.0"),
+                                    false, true, "somehost.example.com.", false);
+    ctx.query_.reset(new Pkt4(DHCPREQUEST, 1234));
+    ctx.callout_handle_ = callout_handle;
+
     // Call allocateLease4. Callouts should be triggered here.
-    Lease4Ptr lease = engine->allocateLease4(subnet_, clientid_, hwaddr_,
-                                             IOAddress("0.0.0.0"),
-                                             false, false, "",
-                                             false, callout_handle,
-                                             old_lease_);
+    Lease4Ptr lease = engine->allocateLease4(ctx);
+
     // Check that we got a lease
     ASSERT_TRUE(lease);
 
@@ -475,6 +582,41 @@ TEST_F(HookAllocEngine4Test, change_lease4_select) {
     EXPECT_EQ(valid_override_, from_mgr->valid_lft_);
 }
 
+// This test checks if lease4_select callout can set the status to next
+// step skip without the engine to retry.
+TEST_F(HookAllocEngine4Test, skip_lease4_select) {
+
+    // Create allocation engine (hook names are registered in its ctor)
+    boost::scoped_ptr<AllocEngine> engine;
+    ASSERT_NO_THROW(engine.reset(new AllocEngine(AllocEngine::ALLOC_ITERATIVE,
+                                                 100, false)));
+    ASSERT_TRUE(engine);
+
+    // Initialize Hooks Manager
+    HookLibsCollection libraries; // no libraries at this time
+    HooksManager::loadLibraries(libraries);
+
+    // Install a callout
+    EXPECT_NO_THROW(HooksManager::preCalloutsLibraryHandle().registerCallout(
+                        "lease4_select", lease4_select_skip_callout));
+
+    CalloutHandlePtr callout_handle = HooksManager::createCalloutHandle();
+
+    AllocEngine::ClientContext4 ctx(subnet_, clientid_, hwaddr_,
+                                    IOAddress("0.0.0.0"),
+                                    false, false, "", false);
+    ctx.query_.reset(new Pkt4(DHCPREQUEST, 1234));
+    ctx.callout_handle_ = callout_handle;
+
+    Lease4Ptr lease = engine->allocateLease4(ctx);
+
+    // Check that we got no lease
+    EXPECT_FALSE(lease);
+
+    // Check no retry was attempted
+    EXPECT_EQ(1, callback_skip_);
+}
+
 }; // namespace test
 }; // namespace dhcp
 }; // namespace isc
