@@ -1,4 +1,4 @@
-// Copyright (C) 2015-2016 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2015-2017 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -8,7 +8,6 @@
 #include <dhcp/libdhcp++.h>
 #include <dhcp/option4_addrlst.h>
 #include <dhcp/option6_addrlst.h>
-#include <dhcp/option_space.h>
 #include <dhcp/option_string.h>
 #include <dhcp/option_int.h>
 #include <dhcp/option_vendor.h>
@@ -383,7 +382,7 @@ GenericHostDataSourceTest::compareOptions(const ConstCfgOptionPtr& cfg1,
             // Options must be represented by the same C++ class derived from
             // the Option class.
             EXPECT_TRUE(typeid(*option1) == typeid(*option2))
-                << "Comapared DHCP options, having option code "
+                << "Compared DHCP options, having option code "
                 << desc1.option_->getType() << " and belonging to the "
                 << space << " option space, are represented "
                 "by different C++ classes: "
@@ -824,7 +823,7 @@ GenericHostDataSourceTest::testMultipleSubnets(int subnets,
         EXPECT_EQ(1000 + i++, (*it)->getIPv4SubnetID());
     }
 
-    // Finally, check that the hosts can be retrived by HW address or DUID
+    // Finally, check that the hosts can be retrieved by HW address or DUID
     ConstHostCollection all_by_id =
         hdsptr_->getAll(id, &host->getIdentifier()[0],
                         host->getIdentifier().size());
@@ -946,7 +945,7 @@ GenericHostDataSourceTest::testSubnetId6(int subnets, Host::IdentifierType id) {
         EXPECT_EQ(i + 1000, from_hds->getIPv6SubnetID());
     }
 
-    // Check that the hosts can all be retrived by HW address or DUID
+    // Check that the hosts can all be retrieved by HW address or DUID
     ConstHostCollection all_by_id = hdsptr_->getAll(id, &host->getIdentifier()[0],
                                                     host->getIdentifier().size());
     ASSERT_EQ(subnets, all_by_id.size());
@@ -1003,6 +1002,46 @@ void GenericHostDataSourceTest::testGetByIPv6(Host::IdentifierType id,
     EXPECT_FALSE(hdsptr_->get6(IOAddress("2001:db8::5"), len));
 }
 
+void GenericHostDataSourceTest::testGetBySubnetIPv6() {
+    // Make sure we have a pointer to the host data source.
+    ASSERT_TRUE(hdsptr_);
+
+    // Let's create a couple of hosts...
+    HostPtr host1 = initializeHost6("2001:db8:1::", Host::IDENT_DUID, true);
+    HostPtr host2 = initializeHost6("2001:db8:2::", Host::IDENT_DUID, true);
+    HostPtr host3 = initializeHost6("2001:db8:3::", Host::IDENT_DUID, true);
+    HostPtr host4 = initializeHost6("2001:db8:4::", Host::IDENT_DUID, true);
+
+    // ... and add them to the data source.
+    ASSERT_NO_THROW(hdsptr_->add(host1));
+    ASSERT_NO_THROW(hdsptr_->add(host2));
+    ASSERT_NO_THROW(hdsptr_->add(host3));
+    ASSERT_NO_THROW(hdsptr_->add(host4));
+
+    // And then try to retrieve them back.
+    ConstHostPtr from_hds1 = hdsptr_->get6(host1->getIPv6SubnetID(),
+                                           IOAddress("2001:db8:1::"));
+    ConstHostPtr from_hds2 = hdsptr_->get6(host2->getIPv6SubnetID(),
+                                           IOAddress("2001:db8:2::"));
+    ConstHostPtr from_hds3 = hdsptr_->get6(host3->getIPv6SubnetID(),
+                                           IOAddress("2001:db8:3::"));
+    ConstHostPtr from_hds4 = hdsptr_->get6(host4->getIPv6SubnetID(),
+                                           IOAddress("2001:db8:4::"));
+
+    // Make sure we got something back.
+    ASSERT_TRUE(from_hds1);
+    ASSERT_TRUE(from_hds2);
+    ASSERT_TRUE(from_hds3);
+    ASSERT_TRUE(from_hds4);
+
+    // Then let's check that what we got seems correct.
+    compareHosts(host1, from_hds1);
+    compareHosts(host2, from_hds2);
+    compareHosts(host3, from_hds3);
+    compareHosts(host4, from_hds4);
+}
+
+
 void GenericHostDataSourceTest::testAddDuplicate6WithSameDUID() {
     // Make sure we have the pointer to the host data source.
     ASSERT_TRUE(hdsptr_);
